/*
 * sample_sequence.c
 *
 *  Created on: Jun 16, 2010
 *      Author: rj185
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <sys/sysmacros.h> /* major() minor() */
#include <signal.h>

#include <apci3120.h>
#include "findboards.h"
#include <apci3120-events.h>

/*
 * Global value of the current fd
 */
int current_fd = -1;

/*
 * Enables to stop the acquisition
 */
void exit_test(int sig)
{
	int error;
	printf("Exit the test \n");

	error = ioctl (current_fd, CMD_APCI3120_StopSequenceAcquisition, NULL);
	printf("Stop sequence returned : %d \n", error);
	if ( error != 0)
	{
		close(current_fd);
		exit(EXIT_FAILURE);
	}

	error = ioctl (current_fd, CMD_APCI3120_ResetBoardInterruptRoutine, NULL);
	printf("ReSet board interrupt routine returned : %d \n", error);
	if ( error != 0)
	{
		close(current_fd);
		exit(EXIT_FAILURE);
	}

	close(current_fd);

	exit(EXIT_SUCCESS);
}


void apci3120_run_test(int fd)
{
	int error;
	int cpt = 0;
	int i = 0;

	// Initialize the elements of the sequence
	struct apci3120_sequence_acquisition_data sequence_data;

	sequence_data.sequence_array_size = 4;
	for ( cpt = 0; cpt < sequence_data.sequence_array_size; cpt++ )
	{
		sequence_data.sequence_channels[cpt] = cpt;
		sequence_data.gain[cpt] = APCI3120_1_GAIN;
		sequence_data.polarity[cpt] = APCI3120_BIPOLAR;
	}

	sequence_data.number_of_acquisition = 16;
	sequence_data.dma_mode = APCI3120_DMA_USED;
	sequence_data.extern_trigger = APCI3120_DISABLE;
	sequence_data.acquisition_mode = APCI3120_SIMPLE_MODUS;
	sequence_data.acquisition_cycle = APCI3120_CONTINUOUS;
	sequence_data.delay_timing = 0;
	sequence_data.acquisition_timing = 32760;

	printf("=> CMD_APCI3120_SetBoardInterruptRoutine \n");
	error = ioctl (fd, CMD_APCI3120_SetBoardInterruptRoutine, NULL);
	printf("Set Board Interrupt routine returned : %d \n", error);

	sleep(1);

	printf("=> CMD_APCI3120_InitAnalogInputSequenceAcquisition\n");
	error = ioctl (fd, CMD_APCI3120_InitAnalogInputSequenceAcquisition, &sequence_data);
	printf("Initialize returned : %d \n", error);
	if ( error != 0)
	{
		close(fd);
		exit(EXIT_FAILURE);
	}

	sleep(1);

	error = ioctl (fd, CMD_APCI3120_StartAnalogInputSequenceAcquisition, NULL);
	printf("Start sequence returned : %d \n", error);
	if ( error != 0)
	{
		close(fd);
		exit(EXIT_FAILURE);
	}

	/*
	 * Handle CTRL+C
	 */
	signal( SIGINT, exit_test);

	/* Empty the event buffer */
	printf("Empty the event buffer \n");
	{
		if ( ioctl (fd,CMD_apci3120_EventEmpty,NULL) != 0 )
		{
			printf("Error CMD_apci3120_EventEmpty \n");
			close(fd);
			exit(EXIT_FAILURE);
		}
	}

	/*
	 * Endless while, only interruptible by CTRL+C
	 */
	while(1)
	{
		apci3120_event_t evt;
		if ( ioctl (fd,CMD_apci3120_EventWait, &evt) != 0 )
		{
			printf("Error CMD_apci3701_EventWait \n");
			close(fd);
			exit(EXIT_FAILURE);
		}
		printf("source: %d\n",evt.source);
		printf("Content : \n");
		for ( i = 0; i < sequence_data.sequence_array_size; i++)
		{
			printf( "Channel[%d] : %d \t", i, evt.param1[i]);
		}
		printf("\n");
	}


}

//--------------------------------------------------------------------------------
int main(int argc, char** argv)
{
	int i=0;
	int card_choice = 0;
	int * apci3120_card_fd;
	unsigned int apci3120_card_number = 0;
	{
		apci3120_card_number = apci_find_boards("apci3120",&apci3120_card_fd);

		printf("total: %d cards\n",apci3120_card_number);
	}
	printf("Hit CTRL+C to exit the test \n");

	// No card
	if ( apci3120_card_number == 0)
	{
		printf("No card found, exit ... \n");
		return 0;
	}
	// JUST one card
	if ( apci3120_card_number == 1)
	{
		current_fd = apci3120_card_fd[0];
		// Execute directly the test on this card
		apci3120_run_test(apci3120_card_fd[0]);
		return 0;
	}
	// More than one card
	else
	{
		printf("Choose the card on which you want to do the test : \n");
		printf("Possible choice : \t");
		for ( i = 0; i< apci3120_card_number; i++)
		{
			printf("%d \t", i);
		}
		printf("\n");
		scanf("%d",&card_choice);

		if ( card_choice < 0 || card_choice >= apci3120_card_number)
		{
			printf("Incorrect card choice, exit ... \n");
			return 0;
		}
		else
		{
			current_fd = apci3120_card_fd[card_choice];
			printf("Running test on card : %d \n", card_choice);
			apci3120_run_test(apci3120_card_fd[card_choice]);
		}
	}
	return 0;
}
